// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.AcroForms;
using GrapeCity.Documents.Text;
using System.Security.Cryptography.X509Certificates;

namespace DsPdfWeb.Demos
{
    // このサンプルは、PDF を生成して署名し(SignDoc サンプルに似たコードを使用します）、
    // インクリメンタルアップデートを使用して（Sign() メソッドを使用する場合のデフォルト）、
    // オリジナルの署名を無効にすることなく、生成された PDF に２番目の証明書で署名します。
    public class SignIncremental
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            // 署名されたドキュメントを読み込みます（SignDoc サンプルに似たコードを使用します）
            doc.Load(CreateAndSignPdf());
            // ２番目の証明書を初期化します。
            var pfxPath = Path.Combine("Resources", "Misc", "JohnDoe.pfx");
            var cert = new X509Certificate2(File.ReadAllBytes(pfxPath), "secret",
                X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);
            var sp2 = new SignatureProperties()
            {
                SignatureBuilder = new Pkcs7SignatureBuilder()
                {
                    CertificateChain = new X509Certificate2[] { cert }
                },
                Location = "DioDocs for PDF サンプルブラウザ",
                SignerName = "作成者1",
                SigningDateTime = Common.Util.TimeNow(),
            };
            // 2番目用のまだ使われていない署名フィールドを探します。
            var sfld2 = doc.AcroForm.Fields["SecondSignature"] as SignatureField;
            // 署名フィールドと署名プロパティを結びつけます。
            sp2.SignatureField = sfld2 ?? throw new Exception("予期せぬエラー：'SecondSignature' フィールドが見つかりません。");
            // 署名して文書を保存します。
            doc.Sign(sp2, stream);

            // ストリームを巻き戻して、作成した文書を別の GcPdfDocument
            // オブジェクトに読み込み、署名を確認します。
            stream.Seek(0, SeekOrigin.Begin);
            var doc2 = new GcPdfDocument();
            doc2.Load(stream);
            foreach (var fld in doc2.AcroForm.Fields)
                if (fld is SignatureField sfld)
                    if (!sfld.Value.VerifySignatureValue())
                        throw new Exception($"署名の検証に失敗しました。フィールド：{sfld.Name}");

            // 終了（生成および署名された文書はすでに 'stream' に保存されています）。
            return doc.Pages.Count;
        }

        // このメソッドは、SignDoc サンプルとほぼ同じですが、
        // ２番目の署名フィールドを追加します（署名はしません）。
        private Stream CreateAndSignPdf()
        {
            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            var tf = new TextFormat() { FontName = "Yu Gothic", FontSize = 12 };
            page.Graphics.DrawString("サンプルコードにて以下に2つの署名をしました。\n" +
                "(※注意：ブラウザのビルトインビューワの中には、署名が表示されないものがあります。)",
                tf, new PointF(72, 72));

            // テスト証明書を初期化します。
            var pfxPath = Path.Combine("Resources", "Misc", "GcPdfTest.pfx");
            var cert = new X509Certificate2(File.ReadAllBytes(pfxPath), "qq",
                X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);
            var sp = new SignatureProperties()
            {
                SignatureBuilder = new Pkcs7SignatureBuilder()
                {
                    CertificateChain = new X509Certificate2[] { cert }
                },
                Location = "DioDocs for PDF サンプルブラウザ",
                SignerName = "DioDocs",
                SigningDateTime = Common.Util.TimeNow(),
            };

            // 署名を保持する署名フィールドを初期化します。
            var sf = new SignatureField();
            sf.Widget.Rect = new RectangleF(72, 72 * 2, 72 * 4, 36);
            sf.Widget.Page = page;
            sf.Widget.BackColor = Color.LightSeaGreen;
            // 文書に署名フィールドを追加します。
            doc.AcroForm.Fields.Add(sf);
            // 署名フィールドと署名プロパティを結びつけます。
            sp.SignatureField = sf;

            // ２番目の署名フィールドを追加します。
            var sf2 = new SignatureField() { Name = "SecondSignature" };
            sf2.Widget.Rect = new RectangleF(72, 72 * 3, 72 * 4, 36);
            sf2.Widget.Page = page;
            sf2.Widget.BackColor = Color.LightYellow;
            // 文書に署名フィールドを追加します。
            doc.AcroForm.Fields.Add(sf2);

            var ms = new MemoryStream();
            doc.Sign(sp, ms);
            return ms;
        }
    }
}
